# Copyright (c) Meta Platforms, Inc. and affiliates.

# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import argparse
import subprocess
import os
import json

from .rust_update import RustUpdate, CHECKOUT_METADATA_FILE
from .build import Builder, BUILD_METADATA_FILE
from .dependency_check import DChecker
from bodhi.client.bindings import BodhiClient
from .mockbuild import Mockbuild
from .create_review import PackageReview
from .request_repo_branch import RequestRepo
from .push import Push

def read_fas_user():
    """ 
    Try to read FAS username 
    """
    if os.path.exists(os.path.expanduser("~/.fedora.upn")):
        with open(os.path.expanduser("~/.fedora.upn"), "r") as f:
            return f.read().replace("\n", "")

    klist_line = "Default principal: "
    koutput = subprocess.check_output("klist", universal_newlines=True).split("\n")
    line = [l for l in koutput if l.startswith(klist_line)][0]
    line = line[len(klist_line) :]
    if line.endswith("@FEDORAPROJECT.ORG"):
        print(f"Fetch FAS username: {line[: line.index('@')]}")
        return line[: line.index("@")]

    raise Exception("Could not determine FAS username")

def refresh_kerberos_ticket():
    print("Refreshing Kerberos ticket")
    output = subprocess.run(["kinit", "-R"], capture_output=True)
    if output.returncode == 0:
        return True
    else:
        output = subprocess.run(["kinit", f"{read_fas_user()}@FEDORAPROJECT.ORG"])
        return output.returncode == 0

def main():
    parser = argparse.ArgumentParser("rust-update-set",
                                     formatter_class=argparse.RawTextHelpFormatter)
    subparsers = parser.add_subparsers(help="Command to execute: update/push/build", dest="command")

    # 'update' command: recursively check out all pacakges need to be updated
    parser_update = subparsers.add_parser("update", help="Command to update all crates provided")
    parser_update.add_argument("pkgs", help="Package name and (optional) version like package:version", default = [], nargs="*")
    parser_update.add_argument("-p", "--patch", help="Modify Cargo.toml when creating spec using rust2rpm", dest="patch", action="store_true", default=False)
    parser_update.add_argument("-b", "--branch", dest="branch", help="Specify release branch to be updated")

    # 'push' command: fork repos and push changes to forked repos
    parser_push = subparsers.add_parser("push", help="Command to push all changes to forked repos")
    parser_push.add_argument("-b", "--branch", dest="branch", help="Specify release branch to be updated")

    # 'copr-build' command: build all packages in Copr project
    parser_copr_build = subparsers.add_parser("copr-build", help="Command to build all packages")
    parser_copr_build.add_argument("--copr-project", help="Copr project name")

    # 'chain-build' command: fedpkg chain-build all packages
    parser_build = subparsers.add_parser("chain-build", help="Command to chain-build all packages using fedpkg")
    parser_build.add_argument("--dry-run", dest="dry_run", default=False, action="store_true", help="Option to do a dry run by printing cmd to be executed")
    parser_build.add_argument("-b", "--branch", dest="branch", help="Specify release branch to be updated")

    # 'check' command: check if all packages provided are safe to update
    parser_check = subparsers.add_parser("check", help="Command to check update compatibility")
    parser_check.add_argument("pkgs", help="cPackage name and (optional) version like package:version", default = [], nargs="*")

    # 'bodhi-check' command: check if all packages provided are safe to update
    parser_check = subparsers.add_parser("bodhi-check", help="Command to check update compatibility for a given bodhi update id")
    parser_check.add_argument("--bodhi-id", help="Bodhi update id, e.g. FEDORA-2022-XXXX")

    # 'mockbuild' command: mockbuild all new packages with cmd 'fedora --release RELEASE mockbuild'
    parser_mockbuild = subparsers.add_parser("mockbuild", help="Command to mockbuild all new packages with release specified")
    parser_mockbuild.add_argument("--release", default="rawhide", help="Release for mockbuild new packages")

    # "create-review" command
    parser_create_review = subparsers.add_parser("create-review", help="Create Fedora review for new packages")

    # "request-repo" command
    parser_request_repos = subparsers.add_parser("request-repos", help="Request repositories for all new packages and compat packages")

    # "request-branch" command
    parser_request_branches = subparsers.add_parser("request-branches", help="Request release branches for all new packages and compat packages")
    parser_request_branches.add_argument("branch", default="")
    parser_request_branches.add_argument("--all-releases", action="store_true", default=False)
    
    args = parser.parse_args()

    if args.command == None:
        print("Unknown or missing command, please use correct command [update] [push] [build] [check]")
    
    checkout_metadata = {"checkout":{}, "dependencies":{}, "compats":[]}
    build_metadata = {}
    if os.path.exists(CHECKOUT_METADATA_FILE):
        with open(CHECKOUT_METADATA_FILE) as f:
            checkout_metadata = json.load(f)
    if os.path.exists(BUILD_METADATA_FILE):
        with open(BUILD_METADATA_FILE) as f:
            build_metadata = json.load(f)

    if args.command == "update":
        if args.branch is not None:
            ru = RustUpdate(crates=[], checkout_metadata=checkout_metadata)
            ru.update_branch(args.branch)
            return
        if len(args.pkgs) == 0:
            print(f"No crates provided, please enter the crates you want to update")
            return
        if not refresh_kerberos_ticket():
            print(f"Fail to refresh_kerberos_ticket")
            return
        crates = []
        for pkg in args.pkgs:
            if pkg.count(":") == 0:
                pkg += ":"
            pkg_name, version = pkg.split(":")
            crates.append((pkg_name[5:], version))
        ru = RustUpdate(crates=crates, checkout_metadata=checkout_metadata, patch=args.patch)
        ru.update()
        return
    
    if args.command == "push":
        if args.branch is not None:
            Push.push_branch(checkout_metadata["checkout"], args.branch)
            return
        Push.push(checkout_metadata)
        return

    if args.command == "copr-build":
        if args.copr_project == None:
            print("Please provide Copr project name for creation or build")
            return
        bd = Builder(
            build_metadata=build_metadata, 
            checkout_metadata=checkout_metadata, 
            project=args.copr_project)
        bd.build()
        return
    
    if args.command == "chain-build":
        if not refresh_kerberos_ticket():
            print(f"Fail to refresh_kerberos_ticket")
            return
        bd = Builder(
            build_metadata=build_metadata, 
            checkout_metadata=checkout_metadata, 
            project="")
        bd.chain_build(dry_run=args.dry_run, branch=args.branch)
        return
    
    if args.command == "check":
        crates = []
        for pkg in args.pkgs:
            if pkg.count(":") == 0:
                pkg += ":"
            pkg_name, version = pkg.split(":")
            crates.append((pkg_name[5:], version))
        ru = RustUpdate(crates, checkout_metadata=checkout_metadata)
        ru.check_cmd(crates)
        return
    
    if args.command == "bodhi-check":
        if args.bodhi_id is None:
            print("Please provide a valid bodhi id using --bodhi-id bodhi_id")
            return
        dc = DChecker()
        dc.bodhi_check(args.bodhi_id)
    
    if args.command == "mockbuild":
        Mockbuild.mockbuild(checkout_metadata["checkout"], args.release)
            
    if args.command == "create-review":
        PackageReview.create_review(checkout_metadata["checkout"])
    
    if args.command == "request-repos":
        RequestRepo.request_repos(checkout_metadata["checkout"], checkout_metadata["compats"])

    if args.command == "request-branches":
        if not args.all_release and args.branch=="":
            print("Please specify branch to request or use --all-releases to request all release branches")
        RequestRepo.request_branches(checkout_metadata["checkout"], checkout_metadata["compats"], branch, all_release)

if __name__ == "__main__":
    main()
